<?php
/**
 * Osa_Form_validation.php -
 *
 * Extends the CI Form_validation class. This allows us to pre-process the values and prepare the data array for
 * DB insert/update.
 *
 * @author $Author: dtong $
 * @version $Id: Osa_Form_validation.php,v 1.3 2012/04/02 05:08:47 dtong Exp $
 * @copyright Copyright (c) 2012, Tiller Software Co., Ltd.
*/

class Osa_Form_validation extends CI_Form_validation {
	private $cms_rules = array();

	function __construct($rules = array()) {
		if ( is_array($rules) && !empty($rules) ) {
			$this->cms_rules = $rules;
		}
		parent::__construct($rules);
	}

	function set_rules($field, $label = '', $rules = '') {
		// Do not add to cms_rules if $field is not an array else it will break.
		// The form_validation code will call set_rules again if passing in an array. This will break the code.
		if ( is_array($field) ) {
			$this->cms_rules = array_merge($this->cms_rules, $field);
		}
		return parent::set_rules($field, $label, $rules);
	}

	// Gets all the fields and their value and put into an array for DB insert or update.
	// It is REQUIRED for new code with all form submit handles/controllers to use this.
	// Old code should be changed to use this when time allows.
	// These are the additional arry indexes we add to the CI array.
	// cms_function - A function to process/convert the submitted value such as a checkbox.
	// cms_dbfield - Use this as the column name instead. Setting this to NULL will skip the field in the return array.
	function cms_db_data() {
		if ( empty($this->cms_rules) ) {
			return FALSE;
		}
		$ret = array();
		// controller is CI.
		$controller = &get_instance();
		foreach ($this->cms_rules as $rule) {
			// if ( !($name = osa_value($rule, 'field', FALSE)) ) {
			if ( !isset($rule['field']) ) {
				continue;
			}
			$name = $rule['field'];
			if ( isset($rule['cms_dbfield']) ) {
				$dbname = $rule['cms_dbfield'];
				if ( !$dbname ) {
					// Setting cms_dbfield to false would skip the db update.
					continue;
				}
			}
			else {
				// If not supplied then use the normal name.
				$dbname = $name;
			}
			$value = set_value($name);
			if ( $function = osa_value($rule, 'cms_function', FALSE) ) {
				// We have a value handle function to modify the value before db update/insert.
				// This is necessary for such data from checkbox etc.
				if ( method_exists($controller, $function)) {
					$value = $controller->$function($value);
				}
				elseif ( function_exists($function) ) {
					$value = $function($value);
				}
			}
			$ret[$dbname] = $value;
		}
		if ( empty($ret) ) {
			return FALSE;
		}
		return $ret;
	}
}