<?php
/**
 * unit_compare.php - Library for unit history comparison.
 *
 * @author $Author: dtong $
 * @version $Id: unit_compare.php,v 1.6 2012/02/20 09:59:14 dtong Exp $
 * @copyright Copyright (c) 2009, Tiller Software Co., Ltd.
*/

if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class unit_compare {
	// Highlight color for data in the template body.
	const COLOR_TEMPLATE_HIGHLIGHT = '#FFDDDD';
	// Make sure the color for css class unit_compare_highlight is similiar to COLOR_TEMPLATE_HIGHLIGHT.
	// CSS classes for the fields outside of the template body such as unit title, description etc.
	const CSS_CLASS_HIGHLIGHT = 'unit_compare_highlight';
	const CSS_CLASS_NORMAL = 'unit_compare_normal';

	static $css_classes =
		array('coursename' => self::CSS_CLASS_NORMAL, 'title' => self::CSS_CLASS_NORMAL,
			   'description' => self::CSS_CLASS_NORMAL, 'type' => self::CSS_CLASS_NORMAL,
			   'startdate' => self::CSS_CLASS_NORMAL, 'enddate' => self::CSS_CLASS_NORMAL);

	function __construct () {
		$this->CI = & get_instance();
		// $this->CI->load->helper('date');
		$this->CI->load->model('compare/compare_model');
		$this->CI->load->library('unit/compare/compare_data');
		osa_load_lang('unit');
		osa_load_lang('general');
		if ( $blocksched_obj = $this->get_blocksched_object() ) {
			$blocksched_obj->add_css_indexes(self::$css_classes, self::CSS_CLASS_NORMAL);
		}
	}

	// For highlighting the unit title, description etc. that are not part of the template
	// body html. This returns an array of css_class names.
	function highlight_title($unitleft , $unitright) {
		$css_classes = self::$css_classes;
		foreach ($css_classes as $key => $dummy_value) {
			$subobj = FALSE;
			$value_left = FALSE;
			$value_right = FALSE;
			if ( !isset($unitleft->$key) || !isset($unitright->$key) ) {
				 if ( isset($unitleft->unit) && isset($unitright->unit) ) {
				 	  $subobj = 'unit';
				 }
			}

			if ( !$subobj ) {
				$value_left = osa_value($unitleft, $key, FALSE);
				$value_right = osa_value($unitright, $key, FALSE);
			}
			else {
				$value_left = osa_value($unitleft->$subobj, $key, FALSE);
				$value_right = osa_value($unitright->$subobj, $key, FALSE);
			}

			if ($value_left != $value_right) {
				 $css_classes[$key] = self::CSS_CLASS_HIGHLIGHT;
			}
		}
		return $css_classes;
	}

	// For highlighting the template data fields.
	function highlight($unitleft, $unitright) {
		$unit = $unitleft;
		// Test unitleft. We need at least one unit here, that can be left or right.
		if ( !isset($unit->unit) || !isset($unit->unit->template) ) {
			 $unit = $unitright;
			 // Tests unitright
			if ( !isset($unit->unit) || !isset($unit->unit->template) ) {
			 	return FALSE;
			}
		}
		if ( !$unit->unit->template ) {
			osa_log(__METHOD__, 'Missing template name.', $unit);
			echo osa_ajaxmsg(lang('gen_internalservererror'));
			exit;
		}
		$modules = $this->get_modules($unit->unit->template);
		$left = CMS_UNIT_COMPARE_LEFT;
		$right= CMS_UNIT_COMPARE_RIGHT;
		$temp = array($left=>$unitleft, $right=>$unitright);
		$color = self::COLOR_TEMPLATE_HIGHLIGHT;
		foreach ($modules as $key => $item) {
			foreach ($item as $type) {
				$this->CI->compare_data->compare_highlight($temp, $type, $color, $key);
			}
		}
		return $temp;
	}

	// Moved here from unit_init.php
	// Gets a unit record. Use $all_flag=TRUE to get archived or deleted units.
	function get_viewdata($unitid, $all_flag=FALSE) {
		$this->CI->load->model('unit/unit_model');
		if ($all_flag===FALSE) {
			// Skips archived and deleted units.
			$unit = $this->CI->unit_model->get_unit($unitid);
		}
		else {
			// Gets units with all unit status including archived and deleted.
			$unit = $this->CI->unit_model->get_unit_any($unitid);
		}
		if ( !$unit ) {
			echo osa_ajaxmsg(lang('gen_internalservererror'));
			exit;
		}
		if ( $blocksched_obj = $this->get_blocksched_object() ) {
			// Add blockswched range to record.
			$blocksched_obj->assign_block_range($blocksched_obj->TYPE_UNIT_PAGE, $unit);
		}
		$course = $this->CI->unit_model->get_course($unitid);
		if ( !$course ) {
			echo osa_ajaxmsg(lang('gen_internalservererror'));
			exit;
		}

		$data = new stdClass();
		$data->coursename = $course->name;
		$data->unit = $unit;
		$this->_get_unittype_lang($unit, $data, $course);
		return $data;
	}

	// Returns the 3rd party blocksched object if it exists and it is enabled.
	function get_blocksched_object() {
		if ( osa_3p_available() && $this->CI->config->item('cms_3p_blocksched_enabled') ) {
			return $this->CI->cms_3p->load_lib(CMS_3P_LIB_BLOCKSCHED, CMS_3P_FOLDER_BLOCKSCHED);
		}
		return FALSE;
	}

	//Tells a template get modules.
   private function get_modules($template) {
   	if ( !$template ){
			osa_log(__METHOD__, 'Missing template name.');
			echo osa_ajaxmsg(lang('gen_internalservererror'));
			exit;
		}
   	$this->CI->load->library('cms/cms_template/cms_template');
		return $this->CI->cms_template->filename_modules($template);
   }

   // Moved here from unit_init.php
   private function _get_unittype_lang($unit, $data, $course, $courseid=FALSE) {
   	$type   = $unit->type;
   	$status = $unit->status;
   	$data->displaytype = TRUE;	// normal:not display , another type: display
   	$data->iscreator   = FALSE;
   	$data->integrated_flag = FALSE;
   	$data->unified_flag    = FALSE;
   	$unit_lang ='';

   	$data->meta_flag = FALSE;
   	$data->change_flag = TRUE;
   	$data->distribute  = FALSE;
   	switch ($type) {
   		case $this->CI->unit_model->TYPE_NORMAL:
   			$unit_lang = lang('unit_normal');
   			break;
   		case $this->CI->unit_model->TYPE_META:
   			$unit_lang = lang('unit_metaunit');
   			break;
   		case $this->CI->unit_model->TYPE_INT_PARENT:
   			$data->integrated_flag = TRUE;
   			$unit_lang = lang('unit_integratedunit');
   			$data->distribute = $this->CI->unit_model->status_distribute($unit);
   			break;
   		case $this->CI->unit_model->TYPE_INT:
   			$data->integrated_flag = TRUE;
   			$unit_lang = lang('unit_integratedunit');
   			break;
   		case $this->CI->unit_model->TYPE_UNI_PARENT:
   			$data->unified_flag = TRUE;
   			$unit_lang = lang('unit_unifiedunit');
   			$data->distribute = $this->CI->unit_model->status_distribute($unit);
   			break;
   		case $this->CI->unit_model->TYPE_UNI:
   			$data->unified_flag = TRUE;
   			$unit_lang = lang('unit_unifiedunit');
   			break;
   	}

   	$data->unittypename= $unit_lang;
   	$data->meta_flag   = $this->_get_unittype_meta($type);

   	if ($courseid != FALSE) {
   		$action = CMS_ACTION_EDIT;
   		if ($courseid != $course->id ) {
   			if ($data->unified_flag==TRUE) {
   				$action = CMS_ACTION_READ;
   			}
   			else {
   				$action = CMS_ACTION_EDIT;
   			}
   		}
   		else {
   			$data->iscreator = TRUE;
   		}
/*
   		$this->load->library('unit/Unitmain',
   		array('action'=>$action, 'unitid'=>$unit->id, 'courseid'=>$unit->courseid));
   		$data->unitbody = $this->unitmain->render($data->meta_flag);
*/
   	}
   }

   // Moved here from unit_init.php
   private function _get_unittype_meta($type) {
   	$result = FALSE;
   	if ($type == $this->CI->unit_model->TYPE_META) {
   		$result = TRUE;
   	}
   	return $result;
   }

}
